mov ax, 0x07C0  ; set up segments
mov ds, ax
mov es, ax
; stack
mov ax, 0x9000  ; address of the stack SS:SP
mov ss, ax      ; SS = 0x9000 (stack segment)
xor sp, sp      ; SP = 0x0000 (stack pointer)

;;;;;;; FUNCTIONS ;;;;;

;;; Reconfigure the timer (100Hz)
mov ax, 0x34
out 0x43, ax
mov ax, 0x9c
out 0x40, ax
mov ax, 0x2e
out 0x40, ax

;;; Hello screen
mov si, hello_msg

printout:
	lodsb
	
	or al, al
	jz wait_a_moment
	
	mov ah, 0x0e
	int 0x10
	
	jmp printout
	

;;; Wait a moment
wait_a_moment:
	mov bx, 200
	call delay

;;; Initialize screen
; this sets the screen to 320x200 4greys
mov ah, 0x00
mov al, 0x05
int 0x10

;;; prepare the ant
	mov ah, 0xc
	mov al, 3
	mov bh, 0
	mov dx, 0
	
	.yloop:
		mov cx, 0
		.xloop:
			int 0x10
			inc cx
			cmp cx, 320
			jb .xloop
		inc dx
		cmp dx, 200
		jb .yloop
	
	; Everything is set up now, we can now go into the loop
	jmp nextgen

;;; calculate next generation
nextgen:
	; A little delay after every 10th step
	mov cx, [cycles_wait]
	cmp cx, 0
	jne .continue
	mov bx, 1
	call delay
	mov cx, 20
	
	.continue:
	dec cx
	mov [cycles_wait], cx
	; Update XY coords
	mov ax, [ant_x]
	mov bx, [ant_y]
	mov dx, [ant_d]
	
	cmp dx, 1
	je .up_xy_0
	cmp dx, 2
	je .up_xy_1
	cmp dx, 3
	je .up_xy_2
	dec ax
	jmp .updatecont
	
	.up_xy_0:
		dec bx
		jmp .updatecont
	.up_xy_1:
		inc ax
		jmp .updatecont
	.up_xy_2:
		inc bx
		jmp .updatecont
		
	.updatecont:
	mov [ant_x], ax
	mov [ant_y], bx
	call fix_ant
	
	mov ah, 0xd
	mov bh, 0
	mov dx, [ant_x]
	mov cx, [ant_y]
	int 0x10
	
	mov ah, 0xc
	cmp al, 3
	jne .black
	; white
		
		mov al, 0
		int 0x10
		mov ax, [ant_d]
		inc ax
		mov [ant_d], ax
		call fix_ant
		jmp nextgen
	
	.black:
		mov al, 3
		int 0x10
		mov ax, [ant_d]
		dec ax
		mov [ant_d], ax
		call fix_ant
		jmp nextgen

;;; fix ant data
fix_ant:
	push ax
	push bx
	push dx
	
	mov bx, [ant_x]
	mov ax, [ant_y]
	mov dx, [ant_d]
	
	cmp ax, 11
	jb .xmax
	cmp ax, 309
	je .xmin
	
	.chky:
		cmp bx, 1
		jb .ymax
		cmp bx, 199
		je .ymin
	
	.chkd:
		cmp dx, 0
		je .dmax
		cmp dx, 5
		je .dmin
		jmp .finish
	
	.xmax:
		mov ax, 308
		jmp .chky
	.xmin:
		mov ax, 11
		jmp .chky
	
	.ymax:
		mov bx, 198
		jmp .chkd
	.ymin:
		mov bx, 1
		jmp .chkd
	
	.dmax:
		mov dx, 4
		jmp .finish
	.dmin:
		mov dx, 1
		jmp .finish
	
	.finish:
		mov [ant_x], bx
		mov [ant_y], ax
		mov [ant_d], dx
	
		pop dx
		pop bx
		pop ax
		ret


;;; Delay  (bx hold number of ticks...)
delay:
	push ax
	push bx
	push dx
	
	mov ah, 0
	int 0x1a
	
	add bx, dx
	
	.timerloop:
		mov ah, 0
		int 0x1a
		cmp dx, bx
		jg .lexit
		jmp .timerloop
		
	.lexit:
		pop dx
		pop bx
		pop ax
		ret


;;;;;;; CONSTANTS AND MEM ;;;;;
hello_msg db '   ..::langtons ant by kevin chabowski::..', 0
ant_x dw 100
ant_y dw 160
ant_d dw 1
cycles_wait dw 20


; MBR signature
times 510-($-$$) db 0
db 0x55
db 0xAA
